from requestium import Session, Keys
import os
import requests
from requests.auth import HTTPBasicAuth
from tqdm import tqdm


class ExistingImport:
    def __init__(
            self,
            import_ids,
            output_dir,
            initial_auth=None,
            hyku_instance='https://dc.utk-hyku-production.notch8.cloud',
            webdriver_path='/usr/local/bin/chromedriver',
    ):
        self.import_ids = import_ids
        self.output = output_dir
        self.s = Session(
            webdriver_path=webdriver_path,
            browser='chrome',
            default_timeout=15,
            webdriver_options={'arguments': ['headless']}
        )
        self.hyku_instance = hyku_instance
        self.initial_auth = initial_auth
        if initial_auth is not None:
            self.s.driver.get(
                f'https://{initial_auth[0]}:{initial_auth[1]}@{hyku_instance.replace("https://", "")}/users/sign_in?locale=en'
            )

    def sign_in_to_hyku(self, username, password):
        print('\nSigning in to Hyku\n')
        self.s.driver.ensure_element_by_xpath("//input[@id='user_email']").send_keys(username, Keys.ENTER)
        self.s.driver.ensure_element_by_xpath("//input[@id='user_password']").send_keys(password, Keys.ENTER)
        return

    def transfer_cookies_to_requests(self):
        self.requests_session = requests.Session()
        for cookie in self.s.driver.get_cookies():
            cookie_dict = {cookie['name']: cookie['value']}
            self.requests_session.cookies.update(cookie_dict)
        self.requests_session.auth = HTTPBasicAuth(self.initial_auth[0], self.initial_auth[1])

    def export_errors(self):
        self.transfer_cookies_to_requests()
        for import_id in tqdm(self.import_ids):
            url = f'{self.hyku_instance}/importers/{import_id}/export_errors'
            response = self.requests_session.get(url)
            if response.status_code == 200:
                file_path = os.path.join(os.getcwd(), self.output,  f'export_errors_{import_id}.csv')
                with open(f"{file_path}", 'wb') as file:
                    file.write(response.content)
        return



from csv import DictReader, DictWriter

class BanishFiles:
    def __init__(self, csv_file):
        self.csv_file = csv_file
        self.csv_contents = self.__read_csv()

    def __read_csv(self):
        with open(self.csv_file, "r") as file:
            reader = DictReader(file)
            return [
                row for row in reader if "MODS" not in row.get("source_identifier") and "POLICY" not in row.get("source_identifier")
            ]

    def write(self, output_file):
        with open(output_file, "w") as file:
            writer = DictWriter(file, fieldnames=self.csv_contents[0].keys())
            writer.writeheader()
            writer.writerows(self.csv_contents)




import csv
import sys

class FixMetadata:
    def __init__(self, titles, remove_columns=False):
        self.titles = titles
        self.remove_columns = remove_columns

    def update_metadata(self, csv_filename):
        keywords = self.titles
        remove_columns = self.remove_columns
        keywords_list = [keyword.lower() for keyword in keywords.split(',')]
        with open(csv_filename, mode='r', newline='') as infile:
            reader = csv.DictReader(infile)
            rows = list(reader)

        attachment_rows = [row for row in rows if row['model'].lower() == 'attachment']
        fileset_rows = [row for row in rows if row['model'].lower() == 'fileset']

        if len(attachment_rows) + len(fileset_rows) != len(rows):
            print("unexpected model in sheet")
            sys.exit(1)

        attachment_rows.sort(key=lambda row: ('obj' in row['title'].lower() or 'preserve' in row['title'].lower(), row['title'].lower()))
        fileset_rows.sort(key=lambda row: ('obj' in row['title'].lower() or 'preserve' in row['title'].lower(), row['title'].lower()))
        rows = attachment_rows + fileset_rows

        with open(csv_filename, mode='w', newline='') as outfile:
            fieldnames = reader.fieldnames
            if remove_columns:
                # Here are the columns that will not get removed when using the remove columns option
                fieldnames = ['source_identifier', 'title', 'model', 'visibility']
            writer = csv.DictWriter(outfile, fieldnames=fieldnames)
            writer.writeheader()
            for row in rows:
                if any(keyword in row['title'].lower() for keyword in keywords_list):
                    row['visibility'] = 'restricted'
                if remove_columns:
                    row = {key: row[key] for key in fieldnames}
                writer.writerow(row)

if __name__ == "__main__":
    print("use this space to test metadata fixes if needed")